<!-- This file was generated by @travetto/doc and should not be modified directly -->
<!-- Please modify https://github.com/travetto/travetto/tree/main/module/web-http-server/DOC.tsx and execute "npx trv doc" to rebuild -->
# Web HTTP Server Support

## Web HTTP Server Support

**Install: @travetto/web-http-server**
```bash
npm install @travetto/web-http-server

# or

yarn add @travetto/web-http-server
```

This module provides basic for running [http](https://nodejs.org/api/http.html) and [https](https://nodejs.org/api/https.html) servers.  It provides support for ssl key generation during development as well.

## Running a Server
By default, the framework provides a default [@CliCommand](https://github.com/travetto/travetto/tree/main/module/cli/src/decorators.ts#L84) for [WebHttpServer](https://github.com/travetto/travetto/tree/main/module/web-http-server/src/types.ts#L10) that will follow default behaviors, and spin up the server. Currently, [Node Web Server](https://github.com/travetto/travetto/tree/main/module/web-node#readme "Node provider for the travetto web module.") is the only module that provides a compatible [WebHttpServer](https://github.com/travetto/travetto/tree/main/module/web-http-server/src/types.ts#L10).

**Terminal: Standard application**
```bash
$ trv web:http

Initialized {
  manifest: {
    main: {
      name: '@travetto-doc/web-http-server',
      folder: './doc-exec'
    },
    workspace: {
      name: '@travetto-doc/web-http-server',
      path: './doc-exec',
      mono: false,
      manager: 'npm',
      type: 'commonjs',
      defaultEnv: 'local'
    }
  },
  runtime: {
    env: 'local',
    debug: false,
    production: false,
    dynamic: false,
    resourcePaths: [
      './doc-exec/resources'
    ],
    profiles: []
  },
  config: {
    sources: [ { priority: 999, source: 'memory://override' } ],
    active: {
      AcceptConfig: { applies: false, types: [] },
      CacheControlConfig: { applies: true },
      CompressConfig: {
        applies: true,
        supportedEncodings: [ 'br', 'gzip', 'identity', 'deflate' ]
      },
      CookieConfig: { applies: true, httponly: true, sameSite: 'lax', path: '/' },
      CorsConfig: { applies: true },
      DecompressConfig: {
        applies: true,
        supportedEncodings: [ 'br', 'gzip', 'deflate', 'identity' ]
      },
      EtagConfig: { applies: true, minimumSize: '10kb' },
      TrustProxyConfig: { applies: true, ips: [] },
      WebBodyConfig: {
        applies: true,
        limit: '1mb',
        parsingTypes: {
          text: 'text',
          'application/json': 'json',
          'application/x-www-form-urlencoded': 'form'
        }
      },
      WebConfig: { defaultMessage: true },
      WebHttpConfig: { port: 3000, bindAddress: '0.0.0.0', ssl: false },
      WebLogConfig: { applies: true, showStackTrace: true }
    }
  }
}
Listening on port { port: 3000 }
```

### Configuration

**Code: Standard Web Http Config**
```typescript
export class WebHttpConfig {
  /**
   * The port to run on
   */
  @EnvVar('WEB_HTTP_PORT')
  port: number = 3000;

  /**
   * The bind address, defaults to 0.0.0.0
   */
  bindAddress: string = '';

  /**
   * Is SSL active
   */
  @EnvVar('WEB_HTTP_SSL')
  ssl?: boolean;

  /**
   * SSL Keys
   */
  @Secret()
  sslKeys?: WebSslKeyPair;

  async postConstruct(): Promise<void> {
    this.ssl ??= !!this.sslKeys;
    this.port = (this.port < 0 ? await NetUtil.getFreePort() : this.port);
    this.bindAddress ||= await NetUtil.getLocalAddress();

    if (!this.ssl) {
      // Clear out keys if ssl is not set
      this.sslKeys = undefined;
    } else if (!this.sslKeys) {
      if (Runtime.production) {
        throw new AppError('Default ssl keys are only valid for development use, please specify a config value at web.ssl.keys');
      }
      this.sslKeys = await WebSslUtil.generateKeyPair();
    } else {
      if (this.sslKeys.key.length < 100) { // We have files or resources
        this.sslKeys.key = (await RuntimeResources.read(this.sslKeys.key, true)).toString('utf8');
        this.sslKeys.cert = (await RuntimeResources.read(this.sslKeys.cert, true)).toString('utf8');
      }
    }
  }
}
```

### Creating a Custom CLI Entry Point
To customize a Web server, you may need to construct an entry point using the [@CliCommand](https://github.com/travetto/travetto/tree/main/module/cli/src/decorators.ts#L84) decorator. This could look like:

**Code: Application entry point for Web Applications**
```typescript
import { Env, ShutdownManager, toConcrete } from '@travetto/runtime';
import { CliCommand } from '@travetto/cli';
import { DependencyRegistry } from '@travetto/di';
import { RootRegistry } from '@travetto/registry';
import { WebHttpServer, WebHttpConfig } from '@travetto/web-http-server';

import './config-override.ts';

@CliCommand({ runTarget: true })
export class SampleApp {

  preMain(): void {
    Env.TRV_ENV.set('prod');
    Env.NODE_ENV.set('production');
  }

  async main() {
    console.log('CUSTOM STARTUP');
    await RootRegistry.init();
    const ssl = await DependencyRegistry.getInstance(WebHttpConfig);
    ssl.ssl = true;

    // Configure server before running
    const instance = await DependencyRegistry.getInstance(toConcrete<WebHttpServer>());
    const { wait, kill } = await instance.serve();
    ShutdownManager.onGracefulShutdown(kill);
    return wait;
  }
}
```

And using the pattern established in the [Command Line Interface](https://github.com/travetto/travetto/tree/main/module/cli#readme "CLI infrastructure for Travetto framework") module, you would run your program using `npx trv web:custom`.

**Terminal: Custom application**
```bash
$ trv web:custom

CUSTOM STARTUP
Initialized {
  manifest: {
    main: {
      name: '@travetto-doc/web-http-server',
      folder: './doc-exec'
    },
    workspace: {
      name: '@travetto-doc/web-http-server',
      path: './doc-exec',
      mono: false,
      manager: 'npm',
      type: 'commonjs',
      defaultEnv: 'local'
    }
  },
  runtime: {
    env: 'prod',
    debug: false,
    production: true,
    dynamic: false,
    resourcePaths: [
      './doc-exec/resources'
    ],
    profiles: []
  },
  config: {
    sources: [
      { priority: 10, source: 'direct' },
      { priority: 999, source: 'memory://override' }
    ],
    active: {
      AcceptConfig: { applies: false, types: [] },
      CacheControlConfig: { applies: true },
      CompressConfig: {
        applies: true,
        supportedEncodings: [ 'br', 'gzip', 'identity', 'deflate' ]
      },
      CookieConfig: { applies: true, httponly: true, sameSite: 'lax', path: '/' },
      CorsConfig: { applies: true },
      DecompressConfig: {
        applies: true,
        supportedEncodings: [ 'br', 'gzip', 'deflate', 'identity' ]
      },
      EtagConfig: { applies: true, minimumSize: '10kb' },
      TrustProxyConfig: { applies: true, ips: [] },
      WebBodyConfig: {
        applies: true,
        limit: '1mb',
        parsingTypes: {
          text: 'text',
          'application/json': 'json',
          'application/x-www-form-urlencoded': 'form'
        }
      },
      WebConfig: { defaultMessage: true },
      WebHttpConfig: { port: 3000, bindAddress: '0.0.0.0', ssl: true },
      WebLogConfig: { applies: true, showStackTrace: true }
    }
  }
}
Listening on port { port: 3000 }
```

## SSL Support
Additionally the framework supports SSL out of the box, by allowing you to specify your public and private keys for the cert.  In dev mode, the framework will also automatically generate a self-signed cert if:
   *  SSL support is configured
   *  [node-forge](https://www.npmjs.com/package/node-forge) is installed
   *  Not running in prod
   *  No keys provided

This is useful for local development where you implicitly trust the cert. 

SSL support can be enabled by setting `web.http.ssl: true` in your config. The key/cert can be specified as string directly in the config file/environment variables.  The key/cert can also be specified as a path to be picked up by [RuntimeResources](https://github.com/travetto/travetto/tree/main/module/runtime/src/resources.ts#L8).
